#include "vymview.h"

#include <iostream>
using namespace std;

#include "branchitem.h"
#include "mainwindow.h"
#include "mapeditor.h"
#include "treeeditor.h"

extern Main *mainWindow;


VymView::VymView(VymModel *m)
{
	model=m;

	// Create TreeView
	treeEditor=new TreeEditor (model);
	//treeEditor->setModel ((QAbstractItemModel*)model);
	//treeEditor->setMinimumWidth (50);

	treeEditor->setColumnWidth (0,150);
	treeEditor->setAnimated (true);

	// FIXME-2 use proxySelModel=treeEditor->selectionModel();
	selModel=new QItemSelectionModel (model);

	//model->setSelectionModel (proxySelModel);
	model->setSelectionModel (selModel);
	treeEditor->setSelectionModel (selModel);

	// Create good old MapEditor
	mapEditor=model->getMapEditor();
	if (!mapEditor) mapEditor=new MapEditor (model);

	// Connect selections

		// Proxymodel changed
		/*
		connect (
			proxySelModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			this,SLOT (changeProxySelection(const QItemSelection &,const QItemSelection &)));
*/

		// Selection in Model changed	
		connect (
			selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			this,SLOT (changeSelection(const QItemSelection &,const QItemSelection &)));

		// Tell MapEditor to update selection
		connect (
			selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			mapEditor,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));

		// FIXME-2 testing, if that reenables updating selbox during animation
		connect (
			model, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			mapEditor,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));

	// Connect data changed signals	
	connect (
		model, SIGNAL (dataChanged(const QModelIndex &, const QModelIndex &)), 
		mapEditor,SLOT (updateData(const QModelIndex &) ) );

	connect (
		model, SIGNAL (sortFilterChanged (const QString &)),
		treeEditor, SLOT (setSortFilter (const QString &) ) );

	connect (
		model, SIGNAL (noteHasChanged(QModelIndex) ),
		mainWindow, SLOT (updateNoteEditor (QModelIndex) ) );
		
	connect (
		model, SIGNAL (expandAll() ),
		this, SLOT (expandAll () ) );
		
	connect (
		model, SIGNAL (expandOneLevel() ),
		this, SLOT (expandOneLevel() ) );
		
	connect (
		model, SIGNAL (collapseOneLevel() ),
		this, SLOT (collapseOneLevel() ) );
		
	connect (
		model, SIGNAL (showSelection() ),
		this, SLOT (showSelection() ) );
		

	mapEditor->setAntiAlias (mainWindow->isAliased());
	mapEditor->setSmoothPixmap(mainWindow->hasSmoothPixmapTransform());

	addWidget (treeEditor);
	addWidget (mapEditor);

	// Set geometry
	QList <int> widths;
	widths<<200;
	widths<<600;
	setSizes(widths);
}

VymView::~VymView()
{
	//cout << "Destructor VymView\n";
}

VymModel* VymView::getModel()
{
	return model;
}

MapEditor* VymView::getMapEditor()
{
	return mapEditor;
}

void VymView::initFocus()
{
	mapEditor->setFocus();
}

void VymView::changeSelection (const QItemSelection &newsel, const QItemSelection &oldsel)
{
	// Notify mainwindow to update satellites like NoteEditor, if needed (model==currenModel...)
	mainWindow->changeSelection (model,newsel,oldsel);	// FIXME-5 maybe connect VymModel <-> MainWindow directly?
	// would require to also get current model in mainWindow

	//cout << "VV::changeSelection   newsel.count="<<newsel.indexes().count()<<endl;

	if (newsel.indexes().count()>0)
	{

	/* FIXME-2 use proxymodel
		proxySelModel->select (
			treeEditor->getProxyModel()->mapSelectionFromSource (newsel),
			QItemSelectionModel::ClearAndSelect );
		*/
		QModelIndex ix=newsel.indexes().first();
		selModel->setCurrentIndex (ix,QItemSelectionModel::ClearAndSelect  );
		treeEditor->setCurrentIndex (ix);
		showSelection();
	}
}

void VymView::changeProxySelection (const QItemSelection &newsel, const QItemSelection &oldsel)
{
	// Notify mainwindow to update satellites, but map selection to 
	// original model first

	//cout << "VV::changeProxySelection   newsel.count="<<newsel.indexes().count()<<endl;
	if (!newsel.indexes().isEmpty())
	{
	/* FIXME-2 need to set current, too
	*/
		proxySelModel->setCurrentIndex (
			newsel.indexes().first(),
			QItemSelectionModel::ClearAndSelect );
	treeEditor->setCurrentIndex (newsel.indexes().first() );
	}

	// Re-emit but map selection first
	selModel->select (
		treeEditor->getProxyModel()->mapSelectionToSource (newsel),
		QItemSelectionModel::ClearAndSelect );

	showSelection();
}

void VymView::expandAll()
{
	treeEditor->expandAll();
}

void VymView::expandOneLevel()
{
	int level=999999;
	int d;
	BranchItem *cur=NULL;
	BranchItem *prev=NULL;
	QModelIndex pix;

	// Find level to expand
	model->nextBranch(cur,prev);
	while (cur) 
	{
		// FIXME-2 use proxy pix=treeEditor->getProxyModel()->mapFromSource (model->index (cur));
		pix=model->index (cur);
		d=cur->depth();
		if (!treeEditor->isExpanded(pix) && d < level)
			level=d;
		model->nextBranch(cur,prev);	
	}

	// Expand all to level
	cur=NULL;
	prev=NULL;
	model->nextBranch(cur,prev);
	while (cur) 
	{
		// FIXME-2 use proxy pix=treeEditor->getProxyModel()->mapFromSource (model->index (cur));
		pix=model->index (cur);
		d=cur->depth();
		if (!treeEditor->isExpanded(pix) && d <= level && cur->branchCount()>0)
			treeEditor->setExpanded(pix,true);
		model->nextBranch(cur,prev);	
	}
	/* FIXME-3 optimize expanding by using flat version of next
	model->nextBranch(cur,prev,false);
	while (cur) 
	{
		cout << "ok: "<<cur->getHeadingStd()<<endl./re/videochristinaprison1_wmvl.wmv
;
		model->nextBranch(cur,prev,false);	
	}
	*/
}

void VymView::collapseOneLevel()
{
	int level=-1;
	int d;
	BranchItem *cur=NULL;
	BranchItem *prev=NULL;
	QModelIndex pix;

	// Find level to collapse
	model->nextBranch(cur,prev);
	while (cur) 
	{
		// FIXME-2 use proxy pix=treeEditor->getProxyModel()->mapFromSource (model->index (cur));
		pix=model->index (cur);
		d=cur->depth();
		if (treeEditor->isExpanded(pix) && d > level)
			level=d;
		model->nextBranch(cur,prev);	
	}

	// collapse all to level
	cur=NULL;
	prev=NULL;
	model->nextBranch(cur,prev);
	while (cur) 
	{
		// FIXME-2 use proxy pix=treeEditor->getProxyModel()->mapFromSource (model->index (cur));
		pix=model->index (cur);
		d=cur->depth();
		if (treeEditor->isExpanded(pix) && d >= level)
			treeEditor->setExpanded(pix,false);
		model->nextBranch(cur,prev);	
	}
}

void VymView::showSelection()
{
	QModelIndex ix=model->getSelectedIndex();
	treeEditor->scrollTo( ix, QAbstractItemView::EnsureVisible);
	mapEditor->scrollTo ( ix);
}

